function CoxData
% Tested on Windows 7, MATLAB 7.12, 
% 
% Usage: run this script in the same directory awith the following three
% spread sheets: CoxThisisTables_appA.xls, CoxThisisTables_appB.xls, 
% CoxMuscleInfo.xls. Matlab will then open the spreadsheets in order
% to automatically read the data.  Modify lines 33 and 34 below to 
% load and plot a different specimen as below:% 
%  3 =Human Larynx 1
%  4 =Human Larynx 2
%  5 =Human Larynx 3
%  6 =Human Larynx 4
%  7 =Human Larynx 5
%  8 =Human Larynx 6
%  9 =Canine Larynx 1
%  10=Canine Larynx 2
%  11=Canine Larynx 3
%
% Output: two figures (TA and CT) are created, showing the origin and 
% insertion of each muscle bundle from the specimen (with dots) in 3-D 
% space. Surfaces are created to approximate the cartilage the muscles
% connect to.  The figures can be rotated to show 3-D relations.
%
% CoxData version A1
%
% For a tutorial and an update to this script, see www.nrld.org, technical
% note 5.
%
% ejh 2012044
% NRLD.org

%%%% change the below to load in data from a different larynx specimen
rowstrt_=4;     % put in here which row to read (3-12)
rowend_=4;      % put in here where to end read 
                %  keep same as above to read only one set of data, but
                %  put in a higher number to read in multiple specimen and
                %  overlay their plots

%% no need to edit below                
                
figure_=5;
fig_=figure(figure_);
clf,
hold on
filename_='CoxMuscleInfo.xls';
rows_=rowstrt_:rowend_;

try         % is EXCEL open?   
    channel = ddeinit('excel',filename_);
    range_=['r' num2str(rowstrt_) 'c1'];
    tmp_= ddereq(channel,range_,[1 1]); % Note the format array, first 1...
    % is default, second 1 is for text (0 for number)  
catch       % open EXCEL
    tmp_=pwd;
    eval(['!' tmp_ '\' filename_ ' &'])
    disp('... waiting for excel to open...')
    pause(1.25),disp(' . '),pause(1.25),disp(' . ')
    pause(1.25),disp(' . '),pause(1.25),disp('...continuing...'),
    
    channel = ddeinit('excel',filename_);
    range_=['r' num2str(rowstrt_) 'c1'];
    tmp_= ddereq(channel,range_,[1 1]); % Note the format array, first 1...
    % is default, second 1 is for text (0 for number)  
end

for n=1:length(rows_)
    range_=['r' num2str(rows_(n)) 'c1'];
    tmp_= ddereq(channel,range_,[1 1]); % Note the format array, first 1...
    % is default, second 1 is for text (0 for number)  
    Subject_=tmp_(1:end-1);   % an extra space was at the end
    
    %% CT muscle
    range_=['r' num2str(rows_(n)) 'c2'];
    tmp_= ddereq(channel,range_,[1 1]); % Note the format array, first 1...
    % is default, second 1 is for text (0 for number)  
    descriptMus_=tmp_(1:end-1);   % an extra space was at the end   
    range_=['r' num2str(rows_(n)) 'c3' ':r' num2str(rows_(n)) 'c8' ];
    tmp_= ddereq(channel,range_);     %columns
    muscledataCT_=getCox_f(tmp_);
    muscledataCT_.musdesc_(n,:)=descriptMus_;
    muscledataCT_.Subject_(n,:)=Subject_;
    muscledataCT_.LR_='L';      %all CTs from cox were left
    
    tmp_=[muscledataCT_.Subject_(n,:) '_' ...
            muscledataCT_.musdesc_(n,:) '_' ...
            '=muscledataCT_;'];
    eval(tmp_)
    name1_{n}={[muscledataCT_.Subject_(n,:) '_' ...
                muscledataCT_.musdesc_(n,:) '_']};
       
    tmp_=name1_{n};
    var_=['xo=' tmp_{1} '.xo_;,' ...
            'yo=' tmp_{1} '.yo_;,' ...
            'zo=' tmp_{1} '.zo_;,'];
    eval(var_)
    var_=['xi=' tmp_{1} '.xi_;,' ...
            'yi=' tmp_{1} '.yi_;,' ...
            'zi=' tmp_{1} '.zi_;,'];
    eval(var_) 
    fig_=figure(figure_);, hold off
    plotBundles_f(xi,yi,zi,xo,yo,zo,60,60,figure_)
    text(xi(1),yi(1),zi(1),[muscledataCT_.Subject_(n,:) ' Thy'])
    text(xo(1),yo(1),zo(1),[muscledataCT_.Subject_(n,:) ' Cri'])
    title('CT muscle from left larynx')    
    
end


figure_=6;
fig_=figure(figure_);
hold on
filename_='CoxMuscleInfo.xls';
rowstrt_=9;
rowend_=9;
rows_=rowstrt_:rowend_;

try         % is EXCEL open?   
    channel = ddeinit('excel',filename_);
    range_=['r' num2str(rowstrt_) 'c1'];
    tmp_= ddereq(channel,range_,[1 1]); % Note the format array, first 1...
    % is default, second 1 is for text (0 for number)  
catch       % open EXCEL
    tmp_=pwd;
    eval(['!' tmp_ '\' filename_ ' &'])
    disp('... waiting for excel to open...')
    pause(1.25),disp(' . '),pause(1.25),disp(' . ')
    pause(1.25),disp(' . '),pause(1.25),disp('...continuing...'),
    
    channel = ddeinit('excel',filename_);
    range_=['r' num2str(rowstrt_) 'c1'];
    tmp_= ddereq(channel,range_,[1 1]); % Note the format array, first 1...
    % is default, second 1 is for text (0 for number)  
end


for n=1:length(rows_)
    range_=['r' num2str(rows_(n)) 'c1'];
    tmp_= ddereq(channel,range_,[1 1]); % Note the format array, first 1...
    % is default, second 1 is for text (0 for number)  
    Subject_=tmp_(1:end-1);   % an extra space was at the end
    
    
    %% TA muscle
    range_=['r' num2str(rows_(n)) 'c9'];
    tmp_= ddereq(channel,range_,[1 1]); % Note the format array, first 1...
    % is default, second 1 is for text (0 for number)  
    descriptMus_=tmp_(1:end-1);   % an extra space was at the end
    range_=['r' num2str(rows_(n)) 'c10' ':r' num2str(rows_(n)) 'c14' ];
    tmp_= ddereq(channel,range_);     %columns
    muscledataTA_=getCox_f(tmp_);
    muscledataTA_.musdesc_(n,:)=descriptMus_;
    muscledataTA_.Subject_(n,:)=Subject_;   
    muscledataTA_.LR_='R';      %all TAs from cox were right
    
   
    tmp_=[muscledataTA_.Subject_(n,:) '_' ...
            muscledataTA_.musdesc_(n,:) '_' ...
            '=muscledataTA_;'];
    eval(tmp_)
    name2_{n}={[muscledataTA_.Subject_(n,:) '_' ...
                muscledataTA_.musdesc_(n,:) '_']};
    
    tmp_=name2_{n};
    var_=['xo=' tmp_{1} '.xo_;,' ...
            'yo=' tmp_{1} '.yo_;,' ...
            'zo=' tmp_{1} '.zo_;,'];
    eval(var_)
    var_=['xi=' tmp_{1} '.xi_;,' ...
            'yi=' tmp_{1} '.yi_;,' ...
            'zi=' tmp_{1} '.zi_;,'];
    eval(var_)   
    fig_=figure(figure_);, hold off    
    plotBundles_f(xi,yi,zi,xo,yo,zo,60,60,figure_)    
    text(xi(1),yi(1),zi(1),[muscledataTA_.Subject_(n,:) ' Ary'])
    text(xo(1),yo(1),zo(1),[muscledataTA_.Subject_(n,:) ' Thy'])
    title('TA muscle from right larynx')
    
end



disp(' '), disp('    COMPLETE     ')





function muscledata=getCox_f(musinfo_)
% getCox
%   This is a function which reads in the muscle data from a Cox file
%   and returns the requested data in 'muscledata' (which is a class). 
%
%   input:  musinfo_    -an array that lines up muscle data as found 
%                        in CoxMuscleInfo and found in the Cox tables
%
%   returns:muscledata.xo   -origin x data array
%                     .yo   -origin y data array
%                     .zo   -origin z data array
%                     .xi   -insertion x data array
%                     .yi   -insertion y data array
%                     .zi   -insertion z data array
%                     .length   -lenght of bundle (array)
%                     .weight   -weight of bundle (array)
%                     .XSarea   -cross sectional area a
%                     .RorO -Recta or Oblique if CT, na otherwise
%
%
%   Eric Hunter, 20050113


fileappA_='CoxThisisTables_appA.xls';
fileappB_='CoxThisisTables_appB.xls';

range_=['r1c1'];
channelA_=IsExcelOpen(fileappA_,range_);
channelB_=IsExcelOpen(fileappB_,range_);

row_=9;

%get length
forever_=0; cnt_=0;
while forever_==0
    range_=['r' num2str(row_+cnt_) 'c' num2str(musinfo_(1))...
            ':r' num2str(row_+cnt_) 'c' num2str(musinfo_(1))];
    tmp_= ddereq(channelA_,range_,[1 1]);   
    if length(str2num(tmp_))<1
        forever_=1;
    else
        cnt_=cnt_+1;
    end
end

for n=1:cnt_
    % get origin
    range_=['r' num2str(row_+n-1) 'c' num2str(musinfo_(1))...
            ':r' num2str(row_+n-1) 'c' num2str(musinfo_(1)+2)];
    tmp_= ddereq(channelA_,range_);
    muscledata.xo_(n)=tmp_(1);
    muscledata.yo_(n)=tmp_(2);
    muscledata.zo_(n)=tmp_(3);    
    % get insersion
    range_=['r' num2str(row_+n-1) 'c' num2str(musinfo_(2))...
            ':r' num2str(row_+n-1) 'c' num2str(musinfo_(2)+2)];
    tmp_= ddereq(channelA_,range_);
    muscledata.xi_(n)=tmp_(1);
    muscledata.yi_(n)=tmp_(2);
    muscledata.zi_(n)=tmp_(3);    
    % get R or O
    if length(musinfo_)==6
        range_=['r' num2str(row_+n-1) 'c' num2str(musinfo_(6))...
                ':r' num2str(row_+n-1) 'c' num2str(musinfo_(6))];
        tmp_= ddereq(channelA_,range_,[1 1]);
        muscledata.RorO_(n,:)=[tmp_(1) tmp_(1)];
    else
        muscledata.RorO_(n,:)=['na'];
    end    
    % get mass
    range_=['r' num2str(row_+n-1) 'c' num2str(musinfo_(3))...
            ':r' num2str(row_+n-1) 'c' num2str(musinfo_(3))];
    tmp_= ddereq(channelB_,range_);
    muscledata.weight_(n)=tmp_;
    % get length
    muscledata.lngth_(n)=sqrt(...
        (muscledata.xo_(n)-muscledata.xi_(n))^2 +...
        (muscledata.yo_(n)-muscledata.yi_(n))^2 +...
        (muscledata.zo_(n)-muscledata.zi_(n))^2  );
    % get Cross Sectional Area
    muscledata.XSarea_(n)=muscledata.weight_(n)/...
        (muscledata.lngth_(n)*0.001043);
    
end

%get CTJ x y z
range_=['r5c' num2str(musinfo_(4)) ':r5c' num2str(musinfo_(4)+2)];
tmp_= ddereq(channelA_,range_);     % Calibration distance for SPL [cm]
muscledata.CTJ=tmp_;
if length(musinfo_)==6  %CT
    
    %get Bottom of Cricoid x y z
    range_=['r6c' num2str(musinfo_(5)) ':r6c' num2str(musinfo_(5)+2)];
    tmp_= ddereq(channelA_,range_);     % Calibration distance for SPL [cm]
    muscledata.BoC=tmp_;
else    %TA
    %get AnteriorCommisure x y z
    range_=['r6c' num2str(musinfo_(5)) ':r6c' num2str(musinfo_(5)+2)];
    tmp_= ddereq(channelA_,range_);     % Calibration distance for SPL [cm]
    muscledata.AC=tmp_;    
end

function plotBundles_f(xi,yi,zi,xo,yo,zo,xres,yres,figure_)
%PLOTBUNDLES_F Plots bundle info from origin and insertion information.
%
%  send arrays xo, yo, zo (origin data) and xi, yi, zi (insertion data) and
%  xres, yres (x and y resolution for surface)
%
%
%   eric.hunter 20050105


if exist('figure_')==0,figure_=100,end
figure(figure_)

ximin = min(xi); yimin = min(yi);
ximax = max(xi); yimax = max(yi); 
xomin = min(xo); yomin = min(yo);
xomax = max(xo); yomax = max(yo); 


% Define the range and spacing of the x- and y-coordinates,
% and then fit them into X and Y
xiv = linspace(ximin, ximax, xres);
yiv = linspace(yimin, yimax, yres);
[Xiinterp,Yiinterp] = meshgrid(xiv,yiv); 

xres=ceil(xres/2);
yres=ceil(yres/2);

xov = linspace(xomin, xomax, xres);
yov = linspace(yomin, yomax, yres);
[Xointerp,Yointerp] = meshgrid(xov,yov); 


% Calculate Z in the X-Y interpolation space, which is an 
% evenly spaced grid:
Ziinterp = griddata(xi,yi,zi,Xiinterp,Yiinterp,'cubic'); 
Zointerp = griddata(xo,yo,zo,Xointerp,Yointerp,'cubic'); 


% Generate the mesh plot (CONTOUR can also be used):
figure(figure_)
hold on
mesh(Xiinterp,Yiinterp,Ziinterp)
mesh(Xointerp,Yointerp,Zointerp)
plot3(xi,yi,zi,'.')
plot3(xo,yo,zo,'.')
for n=1:length(xi)
plot3([xi(n) xo(n)],[yi(n) yo(n)],[zi(n) zo(n)],':')
end
hold off
xlabel X; ylabel Y; zlabel Z;

function channel_=IsExcelOpen(filename_,range_)

try         % is EXCEL open?   
    channel_ = ddeinit('excel',filename_);
    tmp_= ddereq(channel_,range_); 
catch       % open EXCEL
    tmp_=pwd;
    eval(['!' tmp_ '\' filename_ ' &'])
    disp('... waiting for excel to open...')
    pause(1.75),disp('.'),pause(1.75),disp('.'),pause(1.75),disp('.'),   
    channel_ = ddeinit('excel',filename_);
    tmp_= ddereq(channel_,range_); 
    
end

